// 9.806=


struct Light
{
	float3 pos;
	float3 dir;
	float4 ambient;
	float4 diffuse;
	float4 spec;
	float3 att;
	float  spotPower;
	float  range;
};

struct SurfaceInfo
{
	float3 pos;
    float3 normal;
    float4 diffuse;
    float4 spec;
};

float3 ParallelLight(SurfaceInfo v, Light L, float3 eyePos, float shadowFactor)
{
	float3 litColor = float3(0.0f, 0.0f, 0.0f);
 
	float3 lightVec = -L.dir;
	
	litColor += v.diffuse * L.ambient;	
	
	float diffuseFactor = dot(lightVec, v.normal);
	[branch]
	if( diffuseFactor > 0.0f )
	{
		float specPower   = max(v.spec.a, 1.0f);
		float3 toEye      = normalize(eyePos - v.pos);
		float3 reflectVec = reflect(-lightVec, v.normal);
		float specFactor  = pow(max(dot(reflectVec, toEye), 0.0f), specPower);
				
		litColor += shadowFactor * diffuseFactor * v.diffuse * L.diffuse;
		litColor += shadowFactor * specFactor * v.spec * L.spec;
	}
	
	return litColor;
}

float3 PointLight(SurfaceInfo v, Light L, float3 eyePos, float shadowFactor)
{
	float3 litColor = float3(0.0f, 0.0f, 0.0f);
	
	float3 lightVec = L.pos - v.pos;
		
	float d = length(lightVec);
	
	if( d > L.range )
		return float3(0.0f, 0.0f, 0.0f);
		
	lightVec /= d; 
	
	litColor += v.diffuse * L.ambient;	
	
	
	float diffuseFactor = dot(lightVec, v.normal);
	[branch]
	if( diffuseFactor > 0.0f )
	{
		float specPower   = max(v.spec.a, 1.0f);
		float3 toEye      = normalize(eyePos - v.pos);
		float3 reflectVec = reflect(-lightVec, v.normal);
		float specFactor  = pow(max(dot(reflectVec, toEye), 0.0f), specPower);
	
		litColor += shadowFactor * diffuseFactor * v.diffuse * L.diffuse;
		litColor += shadowFactor * specFactor * v.spec * L.spec;
	}
	
	// attenuate
	return litColor / dot(L.att, float3(1.0f, d, d*d));
}

float3 Spotlight(SurfaceInfo v, Light L, float3 eyePos, float shadowFactor)
{
	float3 litColor = PointLight(v, L, eyePos, shadowFactor);
	
	float3 lightVec = normalize(L.pos - v.pos);
	
	float s = pow(max(dot(-lightVec, L.dir), 0.0f), L.spotPower);
	
	return litColor*s;
}

 
 