// 9.806


#include "lighthelper.fx"
 
static const float SHADOW_EPSILON = 0.002f;
static const float SMAP_SIZE = 2048.0f;
static const float SMAP_DX = 1.0f / SMAP_SIZE;

float	fHeightMapScale		= 0.04;
float	fTexelsPerSide		= sqrt(512.0 * 512.0 * 2);

int		nMaxSamples			= 50;
int		nMinSamples			= 4;

sampler Sampler
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};
 
cbuffer cbPerFrame
{
	Light gLight;
	float3 gEyePosW;
};

cbuffer cbPerObject
{
	float4x4 gLightWVP; 
	float4x4 gWorld;
	float4x4 gWVP; 
	float4x4 gTexMtx;
};

Texture2D gDiffuseMap;
Texture2D gSpecMap;
Texture2D gShadowMap;
TextureCube gCubeMap;

SamplerState gShadowSam
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = Clamp;
	AddressV = Clamp;
};	

SamplerState gTriLinearSam
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};

struct VS_IN
{
	float3 posL     : POSITION;
	float3 tangentL : TANGENT;
	float3 normalL  : NORMAL;
	float2 texC     : TEXCOORD;
    float3 tangent   : TANGENT;
};

struct VS_OUT
{
	float4 posH     : SV_POSITION;
    float3 posW     : POSITION;
    float3 tangentW : TANGENT;
    float3 normalW  : NORMAL;
    //float2 texC     : TEXCOORD0;
    float4 projTexC : TEXCOORD1;

    float2 texC     : TEXCOORD;
    //float3 eye    : TEXCOORD1;
    float3 tangent : TEXCOORD2;
};
 
VS_OUT VS(VS_IN vIn)
{
	VS_OUT vOut;
	
	vOut.posW     = mul(float4(vIn.posL, 1.0f), gWorld);
	vOut.tangentW = mul(float4(vIn.tangentL, 0.0f), gWorld);
	vOut.normalW  = mul(float4(vIn.normalL, 0.0f), gWorld);
		
	vOut.posH = mul(float4(vIn.posL, 1.0f), gWVP);
	
	vOut.projTexC = mul(float4(vIn.posL, 1.0f), gLightWVP);
	
	vOut.texC = mul(float4(vIn.texC, 0.0f, 1.0f), gTexMtx);
	
	return vOut;
}

float CalcShadowFactor(float4 projTexC)
{
	projTexC.xyz /= projTexC.w;

	if( projTexC.x < -1.0f || projTexC.x > 1.0f || 
	    projTexC.y < -1.0f || projTexC.y > 1.0f ||
	    projTexC.z < 0.0f )
	    return 0.0f;

	projTexC.x = +0.5f*projTexC.x + 0.5f;
	projTexC.y = -0.5f*projTexC.y + 0.5f;
	
	float depth = projTexC.z;


	float s0 = gShadowMap.Sample(gShadowSam, projTexC.xy).r;
	float s1 = gShadowMap.Sample(gShadowSam, projTexC.xy + float2(SMAP_DX, 0)).r;
	float s2 = gShadowMap.Sample(gShadowSam, projTexC.xy + float2(0, SMAP_DX)).r;
	float s3 = gShadowMap.Sample(gShadowSam, projTexC.xy + float2(SMAP_DX, SMAP_DX)).r;
	
	float result0 = depth <= s0 + SHADOW_EPSILON;
	float result1 = depth <= s1 + SHADOW_EPSILON;
	float result2 = depth <= s2 + SHADOW_EPSILON;
	float result3 = depth <= s3 + SHADOW_EPSILON;	
	
	float2 texelPos = SMAP_SIZE*projTexC.xy;
	
	float2 t = frac( texelPos );
	
	return lerp( lerp(result0, result1, t.x), lerp(result2, result3, t.x), t.y);
}

float4 PS(VS_OUT pIn) : SV_Target
{
	float4 diffuse = gDiffuseMap.Sample( gTriLinearSam, pIn.texC );
	
	float4 spec    = gSpecMap.Sample( gTriLinearSam, pIn.texC );
	
	float3 normalW = normalize(pIn.normalW);
    float shadowFactor = CalcShadowFactor(pIn.projTexC);
    
    SurfaceInfo v = {pIn.posW, normalW, diffuse, spec};
	float3 litColor = ParallelLight(v, gLight, gEyePosW, shadowFactor);

    return float4(litColor, diffuse.a);
}
 
technique10 ShadowTech
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}
